@@ -1,15 +1,15 @@
 from __future__ import annotations
 
 import json
-from typing import Any, Callable
+from typing import Any, Callable, Dict, List, Optional, Union
 
 import requests
 
 from meilisearch.config import Config
 from meilisearch.errors import (
-    MeiliSearchApiError,
-    MeiliSearchCommunicationError,
-    MeiliSearchTimeoutError,
+    MeilisearchApiError,
+    MeilisearchCommunicationError,
+    MeilisearchTimeoutError,
 )
 from meilisearch.version import qualified_version
 
@@ -26,8 +26,8 @@ class HttpRequests:
         self,
         http_method: Callable,
         path: str,
-        body: dict[str, Any] | list[dict[str, Any]] | list[str] | str | None = None,
-        content_type: str | None = None,
+        body: Optional[Union[Dict[str, Any], List[Dict[str, Any]], List[str], str]] = None,
+        content_type: Optional[str] = None,
     ) -> Any:
         if content_type:
             self.headers["Content-Type"] = content_type
@@ -50,9 +50,9 @@ class HttpRequests:
             return self.__validate(request)
 
         except requests.exceptions.Timeout as err:
-            raise MeiliSearchTimeoutError(str(err)) from err
+            raise MeilisearchTimeoutError(str(err)) from err
         except requests.exceptions.ConnectionError as err:
-            raise MeiliSearchCommunicationError(str(err)) from err
+            raise MeilisearchCommunicationError(str(err)) from err
 
     def get(self, path: str) -> Any:
         return self.send_request(requests.get, path)
@@ -60,31 +60,31 @@ class HttpRequests:
     def post(
         self,
         path: str,
-        body: dict[str, Any] | list[dict[str, Any]] | list[str] | str | None = None,
-        content_type: str | None = "application/json",
+        body: Optional[Union[Dict[str, Any], List[Dict[str, Any]], List[str], str]] = None,
+        content_type: Optional[str] = "application/json",
     ) -> Any:
         return self.send_request(requests.post, path, body, content_type)
 
     def patch(
         self,
         path: str,
-        body: dict[str, Any] | list[dict[str, Any]] | list[str] | str | None = None,
-        content_type: str | None = "application/json",
+        body: Optional[Union[Dict[str, Any], List[Dict[str, Any]], List[str], str]] = None,
+        content_type: Optional[str] = "application/json",
     ) -> Any:
         return self.send_request(requests.patch, path, body, content_type)
 
     def put(
         self,
         path: str,
-        body: dict[str, Any] | list[dict[str, Any]] | list[str] | str | None = None,
-        content_type: str | None = "application/json",
+        body: Optional[Union[Dict[str, Any], List[Dict[str, Any]], List[str], str]] = None,
+        content_type: Optional[str] = "application/json",
     ) -> Any:
         return self.send_request(requests.put, path, body, content_type)
 
     def delete(
         self,
         path: str,
-        body: dict[str, Any] | list[dict[str, Any]] | list[str] | None = None,
+        body: Optional[Union[Dict[str, Any], List[Dict[str, Any]], List[str]]] = None,
     ) -> Any:
         return self.send_request(requests.delete, path, body)
 
@@ -100,4 +100,4 @@ class HttpRequests:
             request.raise_for_status()
             return HttpRequests.__to_json(request)
         except requests.exceptions.HTTPError as err:
-            raise MeiliSearchApiError(str(err), request) from err
+            raise MeilisearchApiError(str(err), request) from err
