@@ -1,3 +1,5 @@
+# pylint: disable=too-many-public-methods
+
 from __future__ import annotations
 
 import base64
@@ -6,15 +8,16 @@ import hashlib
 import hmac
 import json
 import re
-from typing import Any
+from typing import Any, Dict, List, Optional, Union
 from urllib import parse
 
 from meilisearch._httprequests import HttpRequests
 from meilisearch.config import Config
-from meilisearch.errors import MeiliSearchError
+from meilisearch.errors import MeilisearchError
 from meilisearch.index import Index
+from meilisearch.models.key import Key, KeysResults
 from meilisearch.models.task import TaskInfo
-from meilisearch.task import cancel_tasks, delete_tasks, get_task, get_tasks, wait_for_task
+from meilisearch.task import TaskHandler
 
 
 class Client:
@@ -25,7 +28,9 @@ class Client:
     Meilisearch and its permissions.
     """
 
-    def __init__(self, url: str, api_key: str | None = None, timeout: int | None = None) -> None:
+    def __init__(
+        self, url: str, api_key: Optional[str] = None, timeout: Optional[int] = None
+    ) -> None:
         """
         Parameters
         ----------
@@ -38,7 +43,9 @@ class Client:
 
         self.http = HttpRequests(self.config)
 
-    def create_index(self, uid: str, options: dict[str, Any] | None = None) -> dict[str, Any]:
+        self.task_handler = TaskHandler(self.config)
+
+    def create_index(self, uid: str, options: Optional[Dict[str, Any]] = None) -> TaskInfo:
         """Create an index.
 
         Parameters
@@ -50,18 +57,18 @@ class Client:
 
         Returns
         -------
-        task:
-            Dictionary containing a task to track the informations about the progress of an asynchronous process.
+        task_info:
+            TaskInfo instance containing information about a task to track the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return Index.create(self.config, uid, options)
 
-    def delete_index(self, uid: str) -> dict[str, Any]:
+    def delete_index(self, uid: str) -> TaskInfo:
         """Deletes an index
 
         Parameters
@@ -71,19 +78,21 @@ class Client:
 
         Returns
         -------
-        task:
+        task_info:
             Dictionary containing a task to track the informations about the progress of an asynchronous process.
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
 
-        return self.http.delete(f"{self.config.paths.index}/{uid}")
+        task = self.http.delete(f"{self.config.paths.index}/{uid}")
 
-    def get_indexes(self, parameters: dict[str, Any] | None = None) -> dict[str, list[Index]]:
+        return TaskInfo(**task)
+
+    def get_indexes(self, parameters: Optional[Dict[str, Any]] = None) -> Dict[str, List[Index]]:
         """Get all indexes.
 
         Parameters
@@ -98,7 +107,7 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         if parameters is None:
@@ -116,7 +125,7 @@ class Client:
         ]
         return response
 
-    def get_raw_indexes(self, parameters: dict[str, Any] | None = None) -> list[dict[str, Any]]:
+    def get_raw_indexes(self, parameters: Optional[Dict[str, Any]] = None) -> List[Dict[str, Any]]:
         """Get all indexes in dictionary format.
 
         Parameters
@@ -131,7 +140,7 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         if parameters is None:
@@ -154,12 +163,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return Index(self.config, uid).fetch_info()
 
-    def get_raw_index(self, uid: str) -> dict[str, Any]:
+    def get_raw_index(self, uid: str) -> Dict[str, Any]:
         """Get the index as a dictionary.
         This index should already exist.
 
@@ -175,7 +184,7 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return self.http.get(f"{self.config.paths.index}/{uid}")
@@ -196,9 +205,33 @@ class Client:
         """
         if uid is not None:
             return Index(self.config, uid=uid)
-        raise Exception("The index UID should not be None")
+        raise ValueError("The index UID should not be None")
 
-    def get_all_stats(self) -> dict[str, Any]:
+    def multi_search(self, queries: List[Dict[str, Any]]) -> Dict[str, List[Dict[str, Any]]]:
+        """Multi-index search.
+
+        Parameters
+        ----------
+        queries:
+            List of dictionaries containing the specified indexes and their search queries
+            https://docs.meilisearch.com/reference/api/search.html#search-in-an-index
+
+        Returns
+        -------
+        results:
+            Dictionary of results for each search query
+
+        Raises
+        ------
+        MeilisearchApiError
+            An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
+        """
+        return self.http.post(
+            f"{self.config.paths.multi_search}",
+            body={"queries": queries},
+        )
+
+    def get_all_stats(self) -> Dict[str, Any]:
         """Get all stats of Meilisearch
 
         Get information about database size and all indexes
@@ -211,12 +244,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return self.http.get(self.config.paths.stat)
 
-    def health(self) -> dict[str, str]:
+    def health(self) -> Dict[str, str]:
         """Get health of the Meilisearch server.
 
         Returns
@@ -226,7 +259,7 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return self.http.get(self.config.paths.health)
@@ -235,11 +268,11 @@ class Client:
         """Get health of the Meilisearch server."""
         try:
             self.health()
-        except MeiliSearchError:
+        except MeilisearchError:
             return False
         return True
 
-    def get_key(self, key_or_uid: str) -> dict[str, Any]:
+    def get_key(self, key_or_uid: str) -> Key:
         """Gets information about a specific API key.
 
         Parameters
@@ -255,12 +288,14 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.get(f"{self.config.paths.keys}/{key_or_uid}")
+        key = self.http.get(f"{self.config.paths.keys}/{key_or_uid}")
 
-    def get_keys(self, parameters: dict[str, Any] | None = None) -> dict[str, Any]:
+        return Key(**key)
+
+    def get_keys(self, parameters: Optional[Dict[str, Any]] = None) -> KeysResults:
         """Gets the Meilisearch API keys.
 
         Parameters
@@ -271,19 +306,21 @@ class Client:
         Returns
         -------
         keys:
-            Dictionary with limit, offset, total and results a list of dictionaries containing the key information.
+            API keys.
             https://docs.meilisearch.com/reference/api/keys.html#get-keys
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         if parameters is None:
             parameters = {}
-        return self.http.get(f"{self.config.paths.keys}?{parse.urlencode(parameters)}")
+        keys = self.http.get(f"{self.config.paths.keys}?{parse.urlencode(parameters)}")
 
-    def create_key(self, options: dict[str, Any]) -> dict[str, Any]:
+        return KeysResults(**keys)
+
+    def create_key(self, options: Dict[str, Any]) -> Key:
         """Creates a new API key.
 
         Parameters
@@ -297,23 +334,25 @@ class Client:
 
         Returns
         -------
-        keys:
+        key:
             The new API key.
             https://docs.meilisearch.com/reference/api/keys.html#get-keys
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(f"{self.config.paths.keys}", options)
+        task = self.http.post(f"{self.config.paths.keys}", options)
 
-    def update_key(self, key_or_uid: str, options: dict[str, Any]) -> dict[str, Any]:
+        return Key(**task)
+
+    def update_key(self, key_or_uid: str, options: Dict[str, Any]) -> Key:
         """Update an API key.
 
         Parameters
         ----------
-        key:
+        key_or_uid:
             The key or the uid of the key for which to update the information.
         options:
             The information to use in creating the key (ex: { 'description': 'Search Key', 'expiresAt': '22-01-01' }). Note that if an
@@ -327,13 +366,15 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         url = f"{self.config.paths.keys}/{key_or_uid}"
-        return self.http.patch(url, options)
+        key = self.http.patch(url, options)
 
-    def delete_key(self, key_or_uid: str) -> dict[str, int]:
+        return Key(**key)
+
+    def delete_key(self, key_or_uid: str) -> int:
         """Deletes an API key.
 
         Parameters
@@ -349,12 +390,14 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.delete(f"{self.config.paths.keys}/{key_or_uid}")
+        response = self.http.delete(f"{self.config.paths.keys}/{key_or_uid}")
 
-    def get_version(self) -> dict[str, str]:
+        return response.status_code
+
+    def get_version(self) -> Dict[str, str]:
         """Get version Meilisearch
 
         Returns
@@ -364,12 +407,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return self.http.get(self.config.paths.version)
 
-    def version(self) -> dict[str, str]:
+    def version(self) -> Dict[str, str]:
         """Alias for get_version
 
         Returns
@@ -379,12 +422,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return self.get_version()
 
-    def create_dump(self) -> dict[str, str]:
+    def create_dump(self) -> TaskInfo:
         """Trigger the creation of a Meilisearch dump.
 
         Returns
@@ -395,12 +438,14 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return self.http.post(self.config.paths.dumps)
+        task = self.http.post(self.config.paths.dumps)
 
-    def swap_indexes(self, parameters: list[dict[str, list[str]]]) -> TaskInfo:
+        return TaskInfo(**task)
+
+    def swap_indexes(self, parameters: List[Dict[str, List[str]]]) -> TaskInfo:
         """Swap two indexes.
 
         Parameters
@@ -416,14 +461,14 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
         return TaskInfo(**self.http.post(self.config.paths.swap, parameters))
 
     def get_tasks(
-        self, parameters: dict[str, Any] | None = None
-    ) -> dict[str, list[dict[str, Any]]]:
+        self, parameters: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, List[Dict[str, Any]]]:
         """Get all tasks.
 
         Parameters
@@ -438,12 +483,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return get_tasks(self.config, parameters=parameters)
+        return self.task_handler.get_tasks(parameters=parameters)
 
-    def get_task(self, uid: int) -> dict[str, Any]:
+    def get_task(self, uid: int) -> Dict[str, Any]:
         """Get one task.
 
         Parameters
@@ -458,12 +503,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return get_task(self.config, uid)
+        return self.task_handler.get_task(uid)
 
-    def cancel_tasks(self, parameters: dict[str, Any]) -> TaskInfo:
+    def cancel_tasks(self, parameters: Dict[str, Any]) -> TaskInfo:
         """Cancel a list of enqueued or processing tasks.
 
         Parameters
@@ -479,12 +524,12 @@ class Client:
 
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return cancel_tasks(self.config, parameters=parameters)
+        return self.task_handler.cancel_tasks(parameters=parameters)
 
-    def delete_tasks(self, parameters: dict[str, Any]) -> TaskInfo:
+    def delete_tasks(self, parameters: Dict[str, Any]) -> TaskInfo:
         """Delete a list of finished tasks.
 
         Parameters
@@ -498,17 +543,17 @@ class Client:
             https://docs.meilisearch.com/reference/api/tasks.html#get-one-task
         Raises
         ------
-        MeiliSearchApiError
+        MeilisearchApiError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return delete_tasks(self.config, parameters=parameters)
+        return self.task_handler.delete_tasks(parameters=parameters)
 
     def wait_for_task(
         self,
         uid: int,
         timeout_in_ms: int = 5000,
         interval_in_ms: int = 50,
-    ) -> dict[str, Any]:
+    ) -> Dict[str, Any]:
         """Wait until Meilisearch processes a task until it fails or succeeds.
 
         Parameters
@@ -516,7 +561,7 @@ class Client:
         uid:
             Identifier of the task to wait for being processed.
         timeout_in_ms (optional):
-            Time the method should wait before raising a MeiliSearchTimeoutError
+            Time the method should wait before raising a MeilisearchTimeoutError
         interval_in_ms (optional):
             Time interval the method should wait (sleep) between requests
 
@@ -527,18 +572,18 @@ class Client:
 
         Raises
         ------
-        MeiliSearchTimeoutError
+        MeilisearchTimeoutError
             An error containing details about why Meilisearch can't process your request. Meilisearch error codes are described here: https://docs.meilisearch.com/errors/#meilisearch-errors
         """
-        return wait_for_task(self.config, uid, timeout_in_ms, interval_in_ms)
+        return self.task_handler.wait_for_task(uid, timeout_in_ms, interval_in_ms)
 
     def generate_tenant_token(
         self,
         api_key_uid: str,
-        search_rules: dict[str, Any] | list[str],
+        search_rules: Union[Dict[str, Any], List[str]],
         *,
-        expires_at: datetime.datetime | None = None,
-        api_key: str | None = None,
+        expires_at: Optional[datetime.datetime] = None,
+        api_key: Optional[str] = None,
     ) -> str:
         """Generate a JWT token for the use of multitenancy.
 
@@ -564,15 +609,15 @@ class Client:
         """
         # Validate all fields
         if api_key == "" or api_key is None and self.config.api_key is None:
-            raise Exception(
+            raise ValueError(
                 "An api key is required in the client or should be passed as an argument."
             )
         if api_key_uid == "" or api_key_uid is None or self._valid_uuid(api_key_uid) is False:
-            raise Exception("An uid is required and must comply to the uuid4 format.")
+            raise ValueError("An uid is required and must comply to the uuid4 format.")
         if not search_rules or search_rules == [""]:
-            raise Exception("The search_rules field is mandatory and should be defined.")
+            raise ValueError("The search_rules field is mandatory and should be defined.")
         if expires_at and expires_at < datetime.datetime.utcnow():
-            raise Exception("The date expires_at should be in the future.")
+            raise ValueError("The date expires_at should be in the future.")
 
         # Standard JWT header for encryption with SHA256/HS256 algorithm
         header = {"typ": "JWT", "alg": "HS256"}
