@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from typing import Optional
+
 
 class Config:
     """
@@ -14,6 +16,7 @@ class Config:
         task = "tasks"
         stat = "stats"
         search = "search"
+        multi_search = "multi-search"
         document = "documents"
         setting = "settings"
         ranking_rules = "ranking-rules"
@@ -31,7 +34,9 @@ class Config:
         faceting = "faceting"
         swap = "swap-indexes"
 
-    def __init__(self, url: str, api_key: str | None = None, timeout: int | None = None) -> None:
+    def __init__(
+        self, url: str, api_key: Optional[str] = None, timeout: Optional[int] = None
+    ) -> None:
         """
         Parameters
         ----------
